%plane_mirror
% Determines the virtual image of an object in a plane mirror.
%
% LAST UPDATED by Andy French April 2017

function plane_mirror
global d

%Mirror height
d.H = 0.4;

%Object centre
x0 = 0.3;
y0 = 0.0;

%Width of object (this scales the image)
w = 0.5;

%Movement on keypress
d.delta = 0.05;

%Fontsize
fsize = 18;

%

%Import bitmap image of object
d.I = imread('Maxwell.jpg');
d.I = double(flipdim(d.I,1))/255;
dim = size(d.I);
Iwidth = dim(2);
Iheight = dim(1);

%Determine x,y coordinates of pixel locations
d.x = linspace(0,w,Iwidth) - w/2 + x0;
d.y = linspace(0,w*Iheight/Iwidth,Iheight) - 0.5*w*Iheight/Iwidth + y0 ;
[d.x,d.y] = meshgrid(d.x,d.y);

%Detrmine most left x coordinate of object
d.xmin = x0-w/2;

%Determine largest and smallest y coordinates of object
d.ymax = 0.5*w*Iheight/Iwidth + y0;
d.ymin = -0.5*w*Iheight/Iwidth + y0;

%Create figure and plot object
d.fig = figure( 'KeyPressFcn',@keypress );
d.Obj = surf(d.x,d.y,-ones(Iheight,Iwidth),d.I);
shading interp
view(2);
hold on;
axis equal
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title('Reflection in a plane mirror','fontsize',fsize);
set(gca,'fontsize',fsize);
xlim([-x0-w,x0+w]);
ylim([-d.delta-d.H,+d.delta+d.H]);
axis manual
box on

%Determine mirror
plot([0,0],[-d.H,d.H],'b')

%Define a particular location on image to draw rays from
d.a = x0;
d.b = 0.5*w*Iheight/Iwidth + y0;

%Determine transformation in mirror
[d.xx,d.yy] = mirror( d.x,d.y );
[d.aa,d.bb] = mirror( d.a,d.b );

%Plot transformed image
d.Imag = surf(d.xx,d.yy,-ones(Iheight,Iwidth),d.I);
shading interp
d.ab = plot( d.a,d.b,'r*' );
d.aabb = plot( d.aa,d.bb,'g*' );

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'uparrow')==1
    d.ymax = d.ymax + d.delta;
    if d.ymax > d.H
        d.ymax =  d.ymax - d.delta;
    else
        d.y = d.y + d.delta;
        d.b = d.b + d.delta;
        d.ymin = d.ymin + d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    d.ymin = d.ymin - d.delta;
    if d.ymin < -d.H
        d.ymin = d.ymin + d.delta;
    else
        d.y = d.y - d.delta;
        d.b = d.b - d.delta;
        d.ymax = d.ymax - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1
    d.xmin = d.xmin - d.delta;
    if ( d.xmin < 0 )
        d.xmin = d.xmin + d.delta;
    else
        d.x = d.x - d.delta;
        d.a = d.a - d.delta;
        update
    end
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    d.x = d.x + d.delta;
    d.a = d.a + d.delta;
    d.xmin = d.xmin + d.delta;
    update
elseif strcmp(get(fig,'currentkey'),'p')==1
    print( d.fig, 'plane mirror.png','-dpng','-r300' );
end

%%

%Update screen
function update
global d

%Determine transformation in mirror
[d.xx,d.yy] = mirror( d.x,d.y );
[d.aa,d.bb] = mirror( d.a,d.b );

%Update object and image
set( d.Obj,'xdata',d.x,'ydata',d.y );
set( d.Imag,'xdata',d.xx,'ydata',d.yy );
set(d.ab,'xdata',d.a,'ydata',d.b);
set(d.aabb,'xdata',d.aa,'ydata',d.bb);
drawnow;

%%

%Transformation in mirror
function [xx,yy] = mirror( x,y )
xx = -x;
yy = y;
bad = x < 0 ;
xx(bad) = NaN;
yy(bad) = NaN;

%End of code